/*
 * Copyright (c) 1993-2003
 *      Texas A&M University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Texas A&M University
 *      and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Developers:
 *     Russell Neeper, David K. Hess, Douglas Lee Schales, David R. Safford,
 *     Wm. Daryl Hawkins
 *
 * $Id: set.c,v 1.1.1.1 2004/01/06 21:09:54 daryl Exp $
 *
 */

#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <netgraph.h>

#include "/usr/src/sys/netgraph/ng_drawbridge.h"
#include "dbmgr.h"
#include "dbmgr.h"
#include "tokens.h"
#include "misc.h"

#define __SET__
#include "text.h"

static void usageSetFlag(void);

/*------------------------------------------------------------*/

static void
usageSetFlag()
{
	fprintf(theOutput, setText[1]);
}

void
usageSet(int verbose)
{
	fprintf(theOutput, "%s%s",
		(verbose) ? setText[0] : setText[0]+1,
		(verbose) ? setText[1] : "");
}

void
usageSetDetailed(int token)
{
	switch (token) {

	case T_MULTICAST:
		fprintf(theOutput, setText[2]);
		break;
	case T_NONIP:
		fprintf(theOutput, setText[3]);
		break;
	case T_OTHERIP:
		fprintf(theOutput, setText[4]);
		break;
	case T_SUSPECTOFF:
		fprintf(theOutput, setText[5]);
		break;
	case T_FRAGICMP:
		fprintf(theOutput, setText[6]);
		break;
	case T_ATTACKICMP:
		fprintf(theOutput, setText[7]);
		break;
	default:
		usageSet(VERBOSE);
		break;
	}
}

/*------------------------------------------------------------*/

void
handleSet()
{
	int i;
	int result;
	char *currTok;
	struct ng_drawbridge_flags flags;
	u_char buffer[2*sizeof(struct ng_mesg) + sizeof(struct ng_drawbridge_flags)];
	struct ng_mesg *const m = (struct ng_mesg *) buffer;
	char path[NG_PATHLEN+1];


	currTok = strtok(NULL, DELIMITERS);
	result = findtoken(currTok);

	currTok = strtok(NULL, DELIMITERS);
	if (!currTok) {
		usageSet(TERSE);
		return;
	}

	switch (result) {

	case T_MULTICAST:
	case T_NONIP:
	case T_OTHERIP:
	case T_SUSPECTOFF:
	case T_FRAGICMP:
	case T_ATTACKICMP:
		if (*currTok == '?') {
			switch (result) {
			case T_MULTICAST:
				fprintf(theOutput, setText[2]);
				break;
			case T_NONIP:
				fprintf(theOutput, setText[3]);
				break;
			case T_OTHERIP:
				fprintf(theOutput, setText[4]);
				break;
			case T_SUSPECTOFF:
				fprintf(theOutput, setText[5]);
				break;
			case T_FRAGICMP:
				fprintf(theOutput, setText[6]);
				break;
			case T_ATTACKICMP:
				fprintf(theOutput, setText[7]);
				break;
			}
			break;
		}
		i = findtoken(currTok);
		if (i == T_FORWARD)
			i = 0;
		else if (i == T_DISCARD)
			i = 1;
		else {
			usageSetFlag();
			break;
		}

		/* Read flags */
		if (NgSendMsg(cskt,"drawbridge:",NGM_DRAWBRIDGE_COOKIE,
		    NGM_DRAWBRIDGE_GET_FLAGS,NULL,0) < 0) {	
		    	dfm_perror("handleSet");	  
			break;
		}
		if(NgRecvMsg(cskt,m,sizeof(buffer),path) < 0) {	
			dfm_perror("handleSet");		
			break;
		}
		else
  		  bcopy((struct ng_drawbridge_flags *)&m->data,&flags,
		        sizeof(struct ng_drawbridge_flags));

		if (result == T_MULTICAST)
			flags.discardMulticast = i;
		else if (result == T_NONIP)
			flags.discardNonIp = i;
		else if (result == T_OTHERIP)
			flags.discardOtherIp = i;
		else if (result == T_SUSPECTOFF)
			flags.discardSuspectOffset = i;
		else if (result == T_FRAGICMP)
			flags.discardFragmentedICMP = i;
		else if (result == T_ATTACKICMP)
			flags.discardAttackICMP = i;
		
		/* Set flags */
		if(NgSendMsg(cskt,"drawbridge:",NGM_DRAWBRIDGE_COOKIE,
			     NGM_DRAWBRIDGE_SET_FLAGS,&flags,sizeof(flags)) < 0) {
			dfm_perror("handleSet");		  
			break;
		}
		break;

	default:
		usageSet(TERSE);
	}
}

/*------------------------------------------------------------*/
