/*
 * Copyright (c) 1993-2003
 *      Texas A&M University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Texas A&M University
 *      and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Developers:
 *     Russell Neeper, David K. Hess, Douglas Lee Schales, David R. Safford,
 *     Wm. Daryl Hawkins
 *
 * $Id: ng_drawbridge.h,v 1.1.1.1 2004/01/06 21:09:54 daryl Exp $
 *
 */

#ifndef _NETGRAPH_DRAWBRIDGE_H_
#define _NETGRAPH_DRAWBRIDGE_H_

#include <sys/types.h>
#include <sys/time.h>
#include <syslog.h>

/* Node type name and magic cookie */
#define NG_DRAWBRIDGE_NODE_TYPE	"drawbridge"
#define NGM_DRAWBRIDGE_COOKIE 	07272002	

/* Hook names */
#define NG_DRAWBRIDGE_HOOK_INSIDE	"inside"
#define NG_DRAWBRIDGE_HOOK_OUTSIDE	"outside"
#define NG_DRAWBRIDGE_HOOK_MIRROR	"mirror"

/* Syslog constants */
#define DEFAULT_SYSL_MASK	0		/* 0x1FFFF for all messages */
#define DEFAULT_SYSL_FACILITY	LOG_LOCAL0

/* Statistics structure for the node */
struct ng_drawbridge_stats {
	u_int64_t	p_insideFiltered;
	u_int64_t	p_outsideFiltered;
	u_int64_t	p_insideRx;
	u_int64_t	p_outsideRx;
	u_int64_t	p_insideTx;
	u_int64_t	p_outsideTx;
	u_int64_t	b_insideFiltered;
	u_int64_t	b_outsideFiltered;
	u_int64_t	b_insideRx;
	u_int64_t	b_outsideRx;
	u_int64_t	b_insideTx;
	u_int64_t	b_outsideTx;
	u_int64_t	f_insideTcpMcast;
	u_int64_t	f_outsideTcpMcast;
	u_int64_t	f_insideUdpMcast;
	u_int64_t	f_outsideUdpMcast;
	u_int64_t	f_insideTcpPort;
	u_int64_t	f_insideTcpPortOverride;
	u_int64_t	f_outsideTcpPort;
	u_int64_t	f_insideUdpPort;
	u_int64_t	f_outsideUdpPort;
	u_int64_t	f_insideIcmpType;
	u_int64_t	f_outsideIcmpType;
	u_int64_t	f_outsideRejectTable;
	u_int64_t	f_outsideSuspectOffset;
	u_int64_t	f_outsideIcmpFrag;
	u_int64_t	f_outsideShortTcpHdr;
	u_int64_t	f_outsideShortUdpHdr;
	u_int64_t	f_outsideSmurfDos;
	u_int64_t	f_outsidePongDos;
	u_int64_t	f_outsideOtherIp;
	u_int64_t	f_outsideNonIp;
	u_int64_t	f_insideAcceptTable;
	u_int64_t	f_insideSuspectOffset;
	u_int64_t	f_insideIcmpFrag;
	u_int64_t	f_insideShortTcpHdr;
	u_int64_t	f_insideShortUdpHdr;
	u_int64_t	f_insideSmurfDos;
	u_int64_t	f_insidePongDos;
	u_int64_t	f_insideOtherIp;
	u_int64_t	f_insideNonIp;
	u_int64_t	insideTCPreset;
	u_int64_t	outsideTCPreset;
	u_int64_t	bridgeEntries;
};

/* Keep this in sync with the above structure definition */
#define NG_DRAWBRIDGE_STATS_TYPE_INFO {					\
	{ "p_insideFiltered",	     	&ng_parse_uint64_type   },      \
	{ "p_outsideFiltered",		&ng_parse_uint64_type   },      \
	{ "p_insideRx", 	     	&ng_parse_uint64_type   },      \
	{ "p_outsideRx",	     	&ng_parse_uint64_type   },      \
	{ "p_insideTx",	     		&ng_parse_uint64_type   },      \
	{ "p_outsideTx",	     	&ng_parse_uint64_type   },      \
	{ "b_insideFiltered",	     	&ng_parse_uint64_type   },      \
	{ "b_outsideFiltered",		&ng_parse_uint64_type   },      \
	{ "b_insideRx",	     		&ng_parse_uint64_type   },      \
	{ "b_outsideRx",	     	&ng_parse_uint64_type   },      \
	{ "b_insideTx",	     		&ng_parse_uint64_type   },      \
	{ "b_outsideTx",	     	&ng_parse_uint64_type   },      \
	{ "f_insideTcpMcast",	     	&ng_parse_uint64_type   },      \
	{ "f_outsideTcpMcast",		&ng_parse_uint64_type   },      \
	{ "f_insideUdpMcast",	     	&ng_parse_uint64_type   },      \
	{ "f_outsideUdpMcast",		&ng_parse_uint64_type   },      \
	{ "f_insideTcpPort",		&ng_parse_uint64_type   },      \
	{ "f_insideTcpPortOverride",	&ng_parse_uint64_type   },      \
	{ "f_outsideTcpPort",	     	&ng_parse_uint64_type   },      \
	{ "f_insideUdpPort",	     	&ng_parse_uint64_type   },      \
	{ "f_outsideUdpPort",	     	&ng_parse_uint64_type   },      \
	{ "f_insideIcmpType",	     	&ng_parse_uint64_type   },      \
	{ "f_outsideIcmpType",		&ng_parse_uint64_type   },      \
	{ "f_outsideRejectTable",	&ng_parse_uint64_type   },      \
	{ "f_outsideSuspectOffset",	&ng_parse_uint64_type   },      \
	{ "f_outsideIcmpFrag",		&ng_parse_uint64_type   },      \
	{ "f_outsideShortTcpHdr",	&ng_parse_uint64_type   },      \
	{ "f_outsideShortUdpHdr",	&ng_parse_uint64_type   },      \
	{ "f_outsideSmurfDos",		&ng_parse_uint64_type   },      \
	{ "f_outsidePongDos",	     	&ng_parse_uint64_type   },      \
	{ "f_outsideOtherIp",	     	&ng_parse_uint64_type   },      \
	{ "f_outsideNonIp",	     	&ng_parse_uint64_type   },      \
	{ "f_insideAcceptTable",	&ng_parse_uint64_type   },      \
	{ "f_insideSuspectOffset",	&ng_parse_uint64_type   },      \
	{ "f_insideIcmpFrag",	     	&ng_parse_uint64_type   },      \
	{ "f_insideShortTcpHdr",	&ng_parse_uint64_type   },      \
	{ "f_insideShortUdpHdr",	&ng_parse_uint64_type   },      \
	{ "f_insideSmurfDos",	     	&ng_parse_uint64_type   },      \
	{ "f_insidePongDos",	     	&ng_parse_uint64_type   },      \
	{ "f_insideOtherIp",	     	&ng_parse_uint64_type   },      \
	{ "f_insideNonIp",	     	&ng_parse_uint64_type   },      \
	{ "insideTCPreset",	     	&ng_parse_uint64_type   },      \
	{ "outsideTCPreset",	     	&ng_parse_uint64_type   },      \
	{ "bridgeEntries",	     	&ng_parse_uint64_type   },      \
	{ NULL }							\
}

/* Drawbridge configuration flags */
struct ng_drawbridge_flags {	
	int8_t discardMulticast;
	int8_t discardNonIp;
	int8_t discardOtherIp;
	int8_t discardSuspectOffset;
	int8_t discardFragmentedICMP;
	int8_t discardAttackICMP;
};

/* Keep this in sync with the above structure definition */
#define NG_DRAWBRIDGE_FLAGS_TYPE_INFO {					\
	{ "discardMulticast",	     	&ng_parse_int8_type   },	\
	{ "discardNonIp",		&ng_parse_int8_type   },	\
	{ "discardOtherIp", 	     	&ng_parse_int8_type   },	\
	{ "discardSuspectOffset",	&ng_parse_int8_type   },	\
	{ "discardFragmentedICMP",	&ng_parse_int8_type   },	\
	{ "discardAttackICMP",	     	&ng_parse_int8_type   },	\
	{ NULL }							\
}

/* Netgraph commands */
enum {
	NGM_DRAWBRIDGE_GET_STATS = 1,		/* get stats */
	NGM_DRAWBRIDGE_CLR_STATS,		/* clear stats */
	NGM_DRAWBRIDGE_GET_CLR_STATS,		/* atomically get and clear stats */
	NGM_DRAWBRIDGE_SET_FLAGS,		/* set flags */
	NGM_DRAWBRIDGE_GET_FLAGS,		/* get flags */
	NGM_DRAWBRIDGE_GET_VERSION,		/* get Drawbridge version */	
	NGM_DRAWBRIDGE_GET_STATUS,		/* get Drawbridge status */
	NGM_DRAWBRIDGE_MANAGE_TABLES,		/* manage tables */
	NGM_DRAWBRIDGE_SET_ADDRESS_TBL,		/* set address table */
	NGM_DRAWBRIDGE_GET_ADDRESS_TBL,		/* get address table */	
	NGM_DRAWBRIDGE_SET_CLASS_TBL,		/* set class table */
	NGM_DRAWBRIDGE_GET_CLASS_TBL,		/* get class table */
	NGM_DRAWBRIDGE_SET_ACCEPT_TBL,		/* set accept table */
	NGM_DRAWBRIDGE_GET_ACCEPT_TBL,		/* get accept table */
	NGM_DRAWBRIDGE_SET_REJECT_TBL,		/* set reject table */
	NGM_DRAWBRIDGE_GET_REJECT_TBL,		/* get reject table */
	NGM_DRAWBRIDGE_SET_OVERRIDE_TBL,	/* set override table */
	NGM_DRAWBRIDGE_GET_OVERRIDE_TBL,	/* get override table */	
	NGM_DRAWBRIDGE_GET_BRIDGE_ENTRY,	/* get bridge table entry */	
	NGM_DRAWBRIDGE_CLR_BRIDGE_TBL,		/* clear bridge table */
};

/* Drawbridge specific defines*/
#define VERSION	"4.0"

#define IS_BROADCAST(address) \
	(*((u_int8_t *) (address)) & 0x01)

#define IS_HW_ADDR_EQU(x1, x2) \
	(((x1)->compare.l == (x2).compare.l) && \
	 ((x1)->compare.w == (x2).compare.w))

#define MALLOC_LEAF \
        ((addrTreeLeaf *)malloc(sizeof(addrTreeLeaf), M_DEVBUF, M_NOWAIT))
 
#define MALLOC_NODE \
        ((addrTreeNode *)malloc(sizeof(addrTreeNode), M_DEVBUF, M_NOWAIT)) 
 
#define MALLOC_CLASS(a) \
        malloc((a) * LIST_TABLE_SIZE, M_DEVBUF, M_NOWAIT);

enum { NO, YES };
enum { DISCARD_PKT, ALLOW_PKT, DISCARD_AND_ANSWER_PKT };
enum { ACTIVATE_NEW, CLEAR_ALL, INIT_NEW, MEM_REPORT };
enum { READ, WRITE, MANAGE };
enum { T_DST_IN, T_DST_OUT, T_SRC_IN, T_SRC_OUT,
       U_DST_IN, U_DST_OUT, U_SRC_IN, U_SRC_OUT,
       I_TYP_IN, I_TYP_OUT, T_RST_IN, T_RST_OUT,
       T_ACCTBL, T_REJTBL };
enum { DEFAULT_ACCESS };
enum { NORMAL, INVERT };
enum { FLAG_MULTICAST=0, FLAG_NONIP, FLAG_OTHERIP, FLAG_SUSPECTOFFSET,
       FLAG_FRAGMENTEDICMP, FLAG_ATTACKICMP, MAX_FLAG };
enum { DB_DROP_PKT, DB_INSIDE_IF, DB_OUTSIDE_IF, DB_BOTH, DB_PROCESS_PKT,
       DB_FORWARD_PKT, DB_DROP_AND_ANSWER_PKT };
       
/* -- The maximum number of bridge table entries must also always -- */
/* -- be a power of two.  In this case 2^13 = 8192 entries.       -- */
#define	A_H_BITS		13
#define MAX_BRIDGE_ADDRESSES	(1 << A_H_BITS)			/* 8192 */
#define ADDRESS_HASH_MASK	(MAX_BRIDGE_ADDRESSES - 1)	/* 0x1FFF */

#define LIST_SIZE 32
#define LIST_TABLE_SIZE	(LIST_SIZE * sizeof(PortListEntry))

/* Network packet defines */
#define SW_IP_OFFMASK 0xFF1F /* byte swapped ip offset mask */

/* -- These should be defined in <net/if_llc.h> but are not -- */
#ifndef LLC_SNAPLEN
#define LLC_SNAPLEN 8
#endif
#ifndef llc_snap
#define llc_snap llc_un.type_snap
#endif

/* Syslog messages */
enum {
	SYSL_UNKNOWN,
	SYSL_COLD_START,	/* cold start */
	SYSL_IN_CLASSD,
	SYSL_OUT_CLASSD,
	SYSL_IN_PORT,
	SYSL_OUT_PORT,		/* only TCP */
	SYSL_IN_TYPE,		/* only srcAddr, dstAddr, and icmp type */
	SYSL_OUT_TYPE,		/* only srcAddr, dstAddr, and icmp type */
	SYSL_IN_REJECT,		/* only protocolNo, srcAddr and dstAddr */
	SYSL_OUT_ACCEPT,	/* only protocolNo, srcAddr and dstAddr */
	SYSL_OUT_OVERRIDE,	/* only TCP */
	SYSL_IN_LENGTH,		/* TCP/UDP-Header maybe not initialized */
	SYSL_OUT_LENGTH,	/* TCP/UDP-Header maybe not initialized */
	SYSL_IN_DOS,		/* denial of service attack packet in */
	SYSL_OUT_DOS,		/* denial of service attack packet out */
	SYSL_IN_PROT,		/* only protocolNo, srcAddr and dstAddr */
	SYSL_OUT_PROT,		/* only protocolNo, srcAddr and dstAddr */
	SYSL_IN_OFFSET,		/* suspect fragment offset */
	SYSL_OUT_OFFSET,	/* suspect fragment offset */
	SYSL_IN_FRAG,		/* fragmented packet */
	SYSL_OUT_FRAG,		/* fragmented packet */
	SYSL_IN_FILTER,		/* no packet information */
	SYSL_OUT_FILTER		/* no packet information */
};

/* Denial of service (DOS) attacks */
enum dos_type {
	DOS_SMURF, DOS_PONG
};

/* Error messages */
enum error_msg {
	NO_ERROR=0,
	ERROR_OUT_OF_RANGE=100,	ERROR_UNKNOWN_CLASS,
	ERROR_INVALID_NETWORK,	ERROR_TABLE_FULL,
	ERROR_NET_NOT_FOUND,	ERROR_NO_MEMORY,
	ERROR_INVALID_OP,	ERROR_IF_DOWN,
	ERROR_GET_HW,		ERROR_GET_IP,
	ERROR_INVALID_IF,	ERROR_SAME_IF,
	ERROR_DIFF_IF_TYPE,	ERROR_BAD_IF_TYPE,
	ERROR_INSIDE_DOWN,	ERROR_OUTSIDE_DOWN,
	ERROR_TWO_IP,		ERROR_NO_IP,
	ERROR_INVALID_MODE,	ERROR_NOT_INITILIZED,
	ERROR_RUNNING,		ERROR_NOT_RUNNING,
	ERROR_MIRROR_DOWN,	ERROR_MIRROR_UNDEFINED,
	ERROR_MIRROR_MODE,
	MAX_ERROR_NUM  /* must be last */
};

/* General Drawbridge structures */

typedef struct _versionReq {
	u_int8_t version[32];
} versionReq;

typedef struct _SyslogMessageEntry {
	const char *message;
	unsigned char priority;
} SyslogMessageEntry;

typedef union _in_addrw {
        u_int32_t s_addr;
        struct {
                u_int8_t s_b[4];
        } s_un_b;
        struct {
                u_int16_t s_w[2];
        } s_un_w;
#define s_b s_un_b.s_b
#define s_w s_un_w.s_w
} in_addr_tw;

typedef union _HardwareAddr {
	u_int8_t bytes[6];
	u_int16_t words[3];
	struct {
		u_int32_t l;
		u_int16_t w;
	} compare;
	struct {
		u_int8_t bytes[6];
	} addr;
} HardwareAddr;

typedef struct _status {
	u_short	running;
	struct timeval startTime;		
} Status;

/* Structures for tables */
typedef struct {
        u_int16_t c[256];
} addrTreeLeaf;

typedef struct _addrTreeNode {
        union {
                struct _addrTreeNode *p;
                addrTreeLeaf *pd;
                u_int16_t c;
        } u[256];
        u_int8_t flg[256];
} addrTreeNode;

typedef union {
        addrTreeNode *p;
        addrTreeLeaf *pd;
        u_int16_t c;
} addrTreePtr;

typedef struct _PortListEntry {
	u_int16_t begin;
	u_int16_t end;
} PortListEntry;

typedef struct _RejAccTableEntry {
	in_addr_tw network;
	u_int16_t bits;
	u_int16_t flag;
} RejAccTableEntry;

typedef struct _OverrideTableEntry {
	in_addr_tw network;
	u_int32_t bits;
	PortListEntry port[LIST_SIZE];
} OverrideTableEntry;

typedef struct {
        u_int32_t		num_classes;
        addrTreeNode		*addrTree;
        PortListEntry		*t_dst_in;
        PortListEntry		*t_dst_out;
        PortListEntry		*t_src_in;
        PortListEntry		*t_src_out;
        PortListEntry		*u_dst_in;
        PortListEntry		*u_dst_out;
        PortListEntry		*u_src_in;
        PortListEntry		*u_src_out;
        PortListEntry		*i_typ_in;
        PortListEntry		*i_typ_out;
	PortListEntry		*t_rst_in;
	PortListEntry		*t_rst_out;
	addrTreeNode		*acceptTree;
	RejAccTableEntry	*acceptTable;
	u_int32_t		accept_bytes;
	addrTreeNode		*rejectTree;
	RejAccTableEntry	*rejectTable;
	u_int32_t		reject_bytes;
	addrTreeNode		*overrideTree;
	OverrideTableEntry	*overrideTable;
	u_int32_t		override_bytes;
	u_int32_t		num_override;
} pointerStruct;

/* Note: it's assumed that this struct will always be 8 bytes long */
typedef struct _HashEntry {
	HardwareAddr address;
	u_int8_t interface;
	u_int8_t valid;
} HashEntry;

typedef struct _bridgeReq {
	HashEntry t;
	u_int32_t index;
} bridgeReq;

typedef struct _portListReq {
	u_int16_t id;
	u_int16_t class;
	PortListEntry *ptr;
	u_int32_t bytes;
} portListReq;   

typedef struct _addrTreeReq {
	int num;
	int newtable;
	in_addr_tw addr[128];
	u_int8_t bits[128];
	u_int16_t class[128];
} addrTreeReq;

typedef struct _manageTablesReq {
	int command;
	int val;
} manageTablesReq;

typedef struct _TCPresetTableReq {
	PortListEntry in[LIST_SIZE];
	PortListEntry out[LIST_SIZE];
} TCPresetTableReq;

typedef struct _rejaccTableReq {
	u_int32_t bytes;
	void *ptr;
	u_int32_t addr;
	u_int flag;
} rejaccTableReq;

typedef struct _overrideTableReq {
	u_int32_t bytes;
	void *ptr;
	u_int32_t addr;
	u_int index;
} overrideTableReq;

#endif /* _NETGRAPH_DRAWBRIDGE_H_ */
